"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Services = require("./services");
const _1 = require("./");
const Express = require("express");
const http = require("http");
const https = require("https");
const bodyParser = require("body-parser");
const Proxy = require("http-proxy");
const events_1 = require("events");
const termsofservice_1 = require("./termsofservice");
class ServerConfiguration {
}
exports.ServerConfiguration = ServerConfiguration;
function DefaultServerConfiguration() {
    return {
        port: 9080,
        hostname: '127.0.0.1',
        services: {
            'UsernamePasswordAuthService': {},
            'RealmManagementService': {},
            'SyncService': {}
        }
    };
}
exports.DefaultServerConfiguration = DefaultServerConfiguration;
function checkConfiguration(config) {
    config = Object.assign(DefaultServerConfiguration(), config);
    if (typeof config.port !== 'number') {
        try {
            config.port = parseInt(config.port);
        }
        catch (error) {
            throw new Error('The port must be a valid number format.');
        }
    }
    return config;
}
class Server extends events_1.EventEmitter {
    constructor(config = {}) {
        super();
        this.version = require('../package.json').version;
        this.onWebSocketUpgrade = (req, socket, head) => {
            for (let proxy of this.proxies) {
                proxy.ws(req, socket, head);
            }
        };
        this.config = checkConfiguration(config);
        this.proxies = [];
        this.services = [];
        this.expressApp = Express();
        this.expressApp.use(bodyParser.json());
        if (this.config.sslOptions) {
            this.httpServer = https.createServer(this.config.sslOptions, this.expressApp);
        }
        else {
            this.httpServer = http.createServer(this.expressApp);
        }
        this.discovery = new _1.Discovery(this);
        this.httpServer.on('upgrade', this.onWebSocketUpgrade);
    }
    get started() {
        return this._started;
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._started) {
                throw new Error('The server has already been started.');
            }
            yield termsofservice_1.termsOfService(this.config);
            let promises = this.services.map(service => service.start() || Promise.resolve());
            for (const name in this.config.services) {
                const service = yield this.addService(name, this.config.services[name]);
                promises.push(service.start() || Promise.resolve());
                if (service["websocketTargetEndpoint"]) {
                    const websocketTargetEndpoint = service["websocketTargetEndpoint"];
                    let proxy = Proxy.createProxy({
                        target: websocketTargetEndpoint,
                        ws: true
                    });
                    this.proxies.push(proxy);
                }
            }
            yield Promise.all(promises);
            yield new Promise((resolve, reject) => {
                this.httpServer.listen(this.config.port, (err) => {
                    if (err)
                        reject(err);
                    else
                        resolve();
                });
            });
            this._started = true;
            this.emit('serverStarted', this);
        });
    }
    shutdown() {
        return __awaiter(this, void 0, void 0, function* () {
            let promises = [];
            for (const service of this.services) {
                promises.push(service.stop() || Promise.resolve());
            }
            yield Promise.all(promises);
            this.services = [];
            yield new Promise((resolve, reject) => {
                this.httpServer.close((err) => {
                    if (err)
                        reject(err);
                    else
                        resolve();
                });
            });
            this._started = false;
            this.emit('serverStopped', this);
        });
    }
    addService(service, config) {
        if (this._started) {
            throw new Error('Services can only be added before starting the server.');
        }
        let instance = typeof (service) === 'string' ?
            new Services[service](config) : service;
        if (instance.httpEndpoints) {
            if (!instance.baseHttpEndpoint)
                throw new Error('baseHttpEndpoint is required when specifying httpEndpoints');
            const another = this.services.find(service => service.baseHttpEndpoint === instance.baseHttpEndpoint);
            if (another)
                throw new Error(`baseHttpEndpoint must be unique, "${another.name}" already uses "${another.baseHttpEndpoint}"`);
            if (!/^\w+$/.test(instance.baseHttpEndpoint))
                throw new Error('baseHttpEndpoint must be an alphanumeric string');
            const router = Express.Router();
            for (let endpoint of instance.httpEndpoints) {
                router[endpoint.method](endpoint.route, function (req, res, next) {
                    return __awaiter(this, void 0, void 0, function* () {
                        try {
                            const handlerResponse = yield endpoint.handler(req);
                            if (handlerResponse["statusCode"]) {
                                res.set(handlerResponse["headers"])
                                    .status(handlerResponse["statusCode"])
                                    .send(handlerResponse["body"]);
                            }
                            else {
                                res.json(handlerResponse || {});
                            }
                        }
                        catch (err) {
                            if (err instanceof _1.JSONError) {
                                res.set('Content-Type', 'application/problem+json')
                                    .status(err.status).json(err);
                            }
                            else {
                                throw new Error(`Internal Server Error: ${err.toString()}`);
                            }
                        }
                    });
                });
            }
            this.expressApp.use('/' + instance.baseHttpEndpoint, router);
        }
        this.services.push(instance);
        instance.discovery = this.discovery;
        return Promise.resolve(instance);
    }
}
exports.Server = Server;
//# sourceMappingURL=server.js.map