"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Realm = require("realm");
const fs = require("fs");
const service_1 = require("../../service");
const shared_1 = require("../../shared");
let userSchema = {
    name: 'AuthUser',
    primaryKey: 'userId',
    properties: {
        userId: 'string',
        isAdmin: { type: 'bool', optional: false }
    }
};
let metadataSchema = {
    name: 'AuthUserMetadata',
    properties: {
        userId: 'string',
        key: 'string',
        value: { type: 'string', optional: true }
    }
};
class AuthService extends service_1.Service {
    get baseHttpEndpoint() { return 'auth'; }
    get httpEndpoints() {
        return [
            { route: '/authenticate', method: 'post', handler: this.authenticate.bind(this) },
            { route: '/', method: 'post', handler: this.authenticateDeprecated.bind(this) },
        ];
    }
    authenticateOrCreateUser(body) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('You must implement the authenticate method in your subclass');
        });
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            this.realm = new Realm({ path: './data/auth.realm', schema: [userSchema, metadataSchema] });
            this.tokenPrivateKey = yield new Promise((resolve, reject) => {
                fs.readFile('./keys/auth.key', (err, data) => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve(data.toString());
                    }
                });
            });
        });
    }
    authenticate(request) {
        return __awaiter(this, void 0, void 0, function* () {
            const authResponse = yield this.authenticateOrCreateUser(request.body);
            if (authResponse) {
                authResponse.authToken = shared_1.signToken({ user_id: authResponse.userId }, this.tokenPrivateKey);
            }
            return authResponse;
        });
    }
    authenticateDeprecated(request) {
        return __awaiter(this, void 0, void 0, function* () {
            if (request.body.path !== undefined) {
                let realmsService = yield this.discovery.find('RealmManagementService');
                if (!realmsService)
                    throw new Error('Unable to contact the Realms service');
                return realmsService.accessToken(request);
            }
            const app_id = request.body["app_id"] || 'io.realm.Auth';
            const oldAuthResponse = yield this.authenticate(request);
            let expires = Math.round(Date.now() / 1000) + 600;
            if (oldAuthResponse) {
                let signedAuthToken = shared_1.signToken({
                    user_id: oldAuthResponse.userId
                }, this.tokenPrivateKey);
                oldAuthResponse.authToken = signedAuthToken;
            }
            oldAuthResponse.refresh_token = {
                token: oldAuthResponse.authToken,
                token_data: {
                    app_id: app_id,
                    identity: oldAuthResponse.userId,
                    is_admin: oldAuthResponse.isAdmin,
                    access: ["refresh"],
                    expires: expires
                }
            };
            return oldAuthResponse;
        });
    }
    getUser(userId) {
        const foundUser = this.realm.objectForPrimaryKey('AuthUser', userId);
        if (foundUser) {
            return {
                userId: foundUser.userId,
                isAdmin: foundUser.isAdmin
            };
        }
        return undefined;
    }
    getUserMetadata(userId) {
        let userMetadata = {};
        this.realm.objects('AuthUserMetadata').filtered('userId = $0', userId).forEach((meta) => {
            userMetadata[meta.key] = meta.value;
        });
        return userMetadata;
    }
    storeUser(userId, isAdmin, metadata) {
        this.realm.write(() => {
            this.realm.create('AuthUser', { userId: userId, isAdmin: isAdmin }, true);
            if (metadata) {
                for (let key in metadata) {
                    this.realm.create('AuthUserMetadata', {
                        userId: userId,
                        key: key,
                        value: metadata[key]
                    }, true);
                }
            }
        });
        return { userId: userId, isAdmin: isAdmin, metadata: this.getUserMetadata(userId) };
    }
    retrieveUser(userId) {
        let user = this.realm.objectForPrimaryKey('AuthUser', userId);
        if (!user)
            return undefined;
        else
            return { userId: userId, isAdmin: user.isAdmin, metadata: this.getUserMetadata(userId) };
    }
}
exports.AuthService = AuthService;
//# sourceMappingURL=authservice.js.map