"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Realm = require("realm");
const service_1 = require("../../service");
const shared_1 = require("../../shared");
const authservice_1 = require("./authservice");
const crypto = require("crypto");
class UsernamePasswordAuthServiceConfig extends service_1.ServiceConfig {
}
exports.UsernamePasswordAuthServiceConfig = UsernamePasswordAuthServiceConfig;
exports.UsernamePasswordConstants = {
    saltWorkIterations: 10000,
    keyLength: 512,
    saltLength: 256,
    digest: "sha256"
};
const PasswordSaltHashSchema = {
    name: 'Password',
    primaryKey: 'userId',
    properties: {
        userId: { type: 'string', optional: false },
        salt: { type: 'string', optional: false },
        hash: { type: 'string', optional: false },
    }
};
class UsernamePasswordAuthService extends authservice_1.AuthService {
    constructor(config) {
        super(config);
        this.config = config;
    }
    start() {
        const _super = name => super[name];
        return __awaiter(this, void 0, void 0, function* () {
            yield _super("start").call(this);
            this.passwordRealm = new Realm({
                path: `./data/password.realm`,
                schema: [PasswordSaltHashSchema]
            });
        });
    }
    hashPassword(clearTextPassword, salt) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                crypto.pbkdf2(clearTextPassword, salt, exports.UsernamePasswordConstants.saltWorkIterations, exports.UsernamePasswordConstants.keyLength, exports.UsernamePasswordConstants.digest, (err, derivedKey) => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve(derivedKey);
                    }
                });
            });
        });
    }
    createSalt(saltWorkFactor) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                crypto.randomBytes(saltWorkFactor, (err, salt) => {
                    if (err) {
                        reject(err);
                    }
                    resolve(salt);
                });
            });
        });
    }
    comparePassword(candidatePassword, salt, hash) {
        return __awaiter(this, void 0, void 0, function* () {
            const hashedCandidatePassword = yield this.hashPassword(candidatePassword, salt);
            return hashedCandidatePassword.equals(hash);
        });
    }
    attemptToRegister(username, password) {
        const _super = name => super[name];
        return __awaiter(this, void 0, void 0, function* () {
            const foundUser = _super("retrieveUser").call(this, username);
            if (foundUser) {
                throw new shared_1.JSONError({
                    detail: `Given account: ${username}`,
                    title: 'The provided credentials are invalid or a user already exists.'
                });
            }
            const salt = yield this.createSalt(exports.UsernamePasswordConstants.saltWorkIterations);
            const hash = yield this.hashPassword(password, salt);
            this.passwordRealm.write(() => {
                this.passwordRealm.create('Password', {
                    salt: salt.toString("base64"),
                    hash: hash.toString("base64"),
                    userId: username
                });
            });
            const storedAuthUser = _super("storeUser").call(this, username, false);
            return {
                userId: storedAuthUser.userId,
                isAdmin: storedAuthUser.isAdmin,
                created: true
            };
        });
    }
    attemptToLogin(username, password) {
        const _super = name => super[name];
        return __awaiter(this, void 0, void 0, function* () {
            const foundUser = _super("retrieveUser").call(this, username);
            if (!foundUser) {
                throw new shared_1.JSONError({
                    detail: `Given account: ${username}`,
                    title: 'The provided credentials are invalid or a user does not exist'
                });
            }
            const foundPassword = this.passwordRealm.objectForPrimaryKey('Password', username);
            const passwordMatch = yield this.comparePassword(password, new Buffer(foundPassword.salt, "base64"), new Buffer(foundPassword.hash, "base64"));
            if (!passwordMatch) {
                throw new shared_1.JSONError({
                    detail: `Given account: ${username}`,
                    title: 'The provided credentials are invalid or a user does not exist'
                });
            }
            return {
                userId: foundUser.userId,
                isAdmin: foundUser.isAdmin,
                created: false
            };
        });
    }
    authenticateOrCreateUser(body) {
        return __awaiter(this, void 0, void 0, function* () {
            let username = "";
            let password = "";
            let register = false;
            if (body["user_info"]) {
                username = body["data"];
                password = body["user_info"]["password"];
                register = body["user_info"]["register"];
            }
            else {
                username = body["username"];
                password = body["password"];
                register = body["register"] || false;
            }
            if (!username) {
                throw new shared_1.JSONError({
                    title: "Your request did not validate because of missing parameter 'username' .",
                    code: 602,
                    path: 'missing-parameters'
                });
            }
            if (!password) {
                throw new shared_1.JSONError({
                    title: "Your request did not validate because of missing parameter 'password'",
                    code: 602,
                    path: 'missing-parameters'
                });
            }
            if (register) {
                return this.attemptToRegister(username, password);
            }
            else {
                return this.attemptToLogin(username, password);
            }
        });
    }
}
exports.UsernamePasswordAuthService = UsernamePasswordAuthService;
//# sourceMappingURL=usernamepassword.js.map